@@ -38,9 +38,9 @@
 #include "neural/network_legacy.h"
 #include "neural/onnx/adapters.h"
 #include "neural/onnx/builder.h"
-#include "neural/shared/activation.h"
-#include "neural/shared/attention_policy_map.h"
-#include "neural/shared/policy_map.h"
+#include "neural/tables/activation_function.h"
+#include "neural/tables/attention_policy_map.h"
+#include "neural/tables/policy_map.h"
 #include "proto/net.pb.h"
 #include "utils/bf16_utils.h"
 #include "utils/exception.h"
@@ -189,8 +189,6 @@ pblczero::TensorProto::DataType Converter::GetDataType() const {
       return pblczero::TensorProto::FLOAT16;
     case WeightsToOnnxConverterOptions::DataType::kBFloat16:
       return pblczero::TensorProto::BFLOAT16;
-    case WeightsToOnnxConverterOptions::DataType::kFloat8E5M2:
-      return pblczero::TensorProto::FLOAT8E5M2;
     default:
       return pblczero::TensorProto::UNDEFINED;
   }
@@ -206,9 +204,6 @@ std::unique_ptr<OnnxConst> Converter::GetWeghtsConverter(
       return std::make_unique<Float16OnnxWeightsAdapter>(weights, dims, order);
     case WeightsToOnnxConverterOptions::DataType::kBFloat16:
       return std::make_unique<BFloat16OnnxWeightsAdapter>(weights, dims, order);
-    case WeightsToOnnxConverterOptions::DataType::kFloat8E5M2:
-      return std::make_unique<Float8E5M2OnnxWeightsAdapter>(weights, dims,
-                                                            order);
   }
   throw Exception("Data type " +
                   std::to_string(static_cast<int>(options_.data_type)) +
@@ -225,9 +220,6 @@ std::unique_ptr<OnnxConst> Converter::GetScalarConverter(float in) {
     case WeightsToOnnxConverterOptions::DataType::kBFloat16:
       return std::make_unique<BFloat16OnnxConst>(
           BFloat16OnnxConst({FP32toBF16(in)}, {1}));
-    case WeightsToOnnxConverterOptions::DataType::kFloat8E5M2:
-      return std::make_unique<Float8E5M2OnnxConst>(
-          Float8E5M2OnnxConst({FP32toFP8E5M2(in)}, {1}));
   }
   throw Exception("Data type " +
                   std::to_string(static_cast<int>(options_.data_type)) +
@@ -237,7 +229,7 @@ std::unique_ptr<OnnxConst> Converter::GetScalarConverter(float in) {
 std::string Converter::StartOptionalBf16Fix(OnnxBuilder* builder,
                                             std::string flow,
                                             std::string name) {
-  if (options_.relax_op_types ||
+  if (options_.opset >= 22 ||
       options_.data_type !=
           WeightsToOnnxConverterOptions::DataType::kBFloat16) {
     return flow;
@@ -247,7 +239,7 @@ std::string Converter::StartOptionalBf16Fix(OnnxBuilder* builder,
 
 std::string Converter::EndOptionalBf16Fix(OnnxBuilder* builder,
                                           std::string flow, std::string name) {
-  if (options_.relax_op_types ||
+  if (options_.opset >= 22 ||
       options_.data_type !=
           WeightsToOnnxConverterOptions::DataType::kBFloat16) {
     return flow;
@@ -258,8 +250,7 @@ std::string Converter::EndOptionalBf16Fix(OnnxBuilder* builder,
 
 std::string Converter::MakeMish(OnnxBuilder* builder, const std::string& input,
                                 const std::string& name) {
-  if (!options_.alt_mish || options_.opset < 9 ||
-      options_.data_type != WeightsToOnnxConverterOptions::DataType::kFloat32) {
+  if (!options_.alt_mish || options_.opset < 9) {
     std::string flow = input;
     flow = StartOptionalBf16Fix(builder, flow, name);
     if (options_.opset >= 18) {
@@ -271,20 +262,31 @@ std::string Converter::MakeMish(OnnxBuilder* builder, const std::string& input,
     flow = builder->Tanh(name + "/tanh", flow);
     return builder->Mul(name, flow, input);
   } else {
+    auto in = input;
+    if (options_.data_type !=
+        WeightsToOnnxConverterOptions::DataType::kFloat32) {
+      in = builder->Cast(name + "/to_float", in,
+                         pblczero::TensorProto::FLOAT);
+    }
     const OnnxConst& two =
         static_cast<const OnnxConst&>(FloatOnnxConst({2.0f}, {1}));
     const OnnxConst& zero =
         static_cast<const OnnxConst&>(FloatOnnxConst({0.0f}, {1}));
-    auto e = builder->Exp(name + "/exp", input);
+    auto e = builder->Exp(name + "/exp", in);
     auto flow = builder->Add(name + "/e+2", e, two);
     auto n = builder->Mul(name + "/n", e, flow);
     flow = builder->Add(name + "/n+2", n, two);
-    auto d = builder->Div(name + "/d", input, flow);
+    auto d = builder->Div(name + "/d", in, flow);
     auto f = builder->Mul(name + "/n*d", n, d);
     flow = builder->Mul(name + "/2*d", d, two);
-    auto t = builder->Sub(name + "/in-2*d", input, flow);
-    flow = builder->Greater(name + "/compare", input, zero);
-    return builder->Where(name, flow, t, f);
+    auto t = builder->Sub(name + "/in-2*d", in, flow);
+    flow = builder->Greater(name + "/compare", in, zero);
+    flow = builder->Where(name, flow, t, f);
+    if (options_.data_type !=
+        WeightsToOnnxConverterOptions::DataType::kFloat32) {
+      flow = builder->Cast(name + "/to_data_type", flow, GetDataType());
+    }
+    return flow;
   }
 }
 
@@ -360,7 +362,7 @@ std::string Converter::MakeConvBlock(
     const std::string& name, const MultiHeadWeights::SEunit* seunit,
     const std::string& mixin, bool activation, int filters) {
   auto flow = input;
-  if (!options_.relax_op_types &&
+  if (options_.opset < 22 &&
       options_.data_type ==
           WeightsToOnnxConverterOptions::DataType::kBFloat16) {
     flow =
@@ -475,8 +477,10 @@ std::string Converter::MakeLayerNorm(OnnxBuilder* builder,
   if (!options_.alt_layernorm) {
     return builder->LayerNormalization(name, input, gammas, betas, 1, eps);
   }
-  auto in =
-      builder->Cast(name + "/to_float", input, pblczero::TensorProto::FLOAT);
+  auto in = input;
+  if (GetDataType() != pblczero::TensorProto::FLOAT) {
+    in = builder->Cast(name + "/to_float", in, pblczero::TensorProto::FLOAT);
+  }
   auto flow = builder->ReduceMean(name + "/mean", in, {1});
   in = builder->Sub(name + "/centered", in, flow);
   flow = builder->Mul(name + "/squared", in, in);
@@ -487,7 +491,9 @@ std::string Converter::MakeLayerNorm(OnnxBuilder* builder,
   flow = builder->Sqrt(name + "/std", flow);
   flow = builder->Reciprocal(name + "/inv_std", flow);
   flow = builder->Mul(name + "/normalized", in, flow);
-  flow = builder->Cast(name + "/to_data_type", flow, GetDataType());
+  if (GetDataType() != pblczero::TensorProto::FLOAT) {
+    flow = builder->Cast(name + "/to_data_type", flow, GetDataType());
+  }
   flow = builder->Mul(name + "/gammas", flow, gammas);
   flow = builder->Add(name + "/betas", flow, betas);
   return flow;
@@ -1094,7 +1100,7 @@ void Converter::MakeMovesLeftHead(pblczero::OnnxModel* onnx,
 
 void Converter::GenerateOnnx(pblczero::OnnxModel* onnx) {
   MultiHeadWeights weights(src_.weights());
-  OnnxBuilder builder(options_.opset);
+  OnnxBuilder builder(options_.opset, options_.ir);
 
   if (GetDataType() == pblczero::TensorProto::FLOAT16) {
     onnx->set_data_type(pblczero::OnnxModel::FLOAT16);
@@ -1221,11 +1227,6 @@ void Converter::Convert(pblczero::Net* dst) {
     throw Exception("The network already has ONNX section.");
   }
   CheckSrcFormat(src_.format().network_format());
-  if (!options_.relax_op_types &&
-      options_.data_type ==
-          WeightsToOnnxConverterOptions::DataType::kFloat8E5M2) {
-    throw Exception("FLOAT8 operation is not supported by the generated ONNX.");
-  }
 
   CopyGenericFields(dst);
   GenerateOnnx(dst->mutable_onnx_model());
@@ -1238,9 +1239,8 @@ WeightsToOnnxConverterOptions::StringToDataType(const std::string& s) {
   if (s == "f32") return DataType::kFloat32;
   if (s == "f16") return DataType::kFloat16;
   if (s == "bf16") return DataType::kBFloat16;
-  if (s == "f8e5m2") return DataType::kFloat8E5M2;
   throw Exception("Invalid data type: [" + s +
-                  "]. Only f32, f16, bf16 and f8e5m2 are supported.");
+                  "]. Only f32, f16 and bf16 are supported.");
 }
 
 pblczero::Net ConvertWeightsToOnnx(
