@@ -30,9 +30,10 @@
 #include <fstream>
 
 #include "chess/pgn.h"
-#include "mcts/search.h"
-#include "mcts/stoppers/factory.h"
-#include "neural/factory.h"
+#include "neural/memcache.h"
+#include "neural/shared_params.h"
+#include "search/classic/search.h"
+#include "search/classic/stoppers/factory.h"
 #include "selfplay/game.h"
 #include "selfplay/multigame.h"
 #include "utils/optionsparser.h"
@@ -110,10 +111,9 @@ void SelfPlayTournament::PopulateOptions(OptionsParser* options) {
     dict->AddSubdict("black")->AddAliasDict(&options->GetOptionsDict("black"));
   }
 
-  NetworkFactory::PopulateOptions(options);
+  SharedBackendParams::Populate(options);
   options->Add<IntOption>(kThreadsId, 1, 8) = 1;
-  options->Add<IntOption>(kNNCacheSizeId, 0, 999999999) = 2000000;
-  SearchParams::Populate(options);
+  classic::SearchParams::Populate(options);
 
   options->Add<BoolOption>(kShareTreesId) = true;
   options->Add<IntOption>(kTotalGamesId, -2, 999999) = -1;
@@ -139,35 +139,33 @@ void SelfPlayTournament::PopulateOptions(OptionsParser* options) {
   SelfPlayGame::PopulateUciParams(options);
 
   auto defaults = options->GetMutableDefaultsOptions();
-  defaults->Set<int>(SearchParams::kMiniBatchSizeId, 32);
-  defaults->Set<float>(SearchParams::kCpuctId, 1.2f);
-  defaults->Set<float>(SearchParams::kCpuctFactorId, 0.0f);
-  defaults->Set<float>(SearchParams::kPolicySoftmaxTempId, 1.0f);
-  defaults->Set<int>(SearchParams::kMaxCollisionVisitsId, 1);
-  defaults->Set<int>(SearchParams::kMaxCollisionEventsId, 1);
-  defaults->Set<int>(SearchParams::kCacheHistoryLengthId, 7);
-  defaults->Set<bool>(SearchParams::kOutOfOrderEvalId, false);
-  defaults->Set<float>(SearchParams::kTemperatureId, 1.0f);
-  defaults->Set<float>(SearchParams::kNoiseEpsilonId, 0.25f);
-  defaults->Set<float>(SearchParams::kFpuValueId, 0.0f);
-  defaults->Set<std::string>(SearchParams::kHistoryFillId, "no");
-  defaults->Set<std::string>(NetworkFactory::kBackendId, "multiplexing");
-  defaults->Set<bool>(SearchParams::kStickyEndgamesId, false);
-  defaults->Set<bool>(SearchParams::kTwoFoldDrawsId, false);
-  defaults->Set<int>(SearchParams::kTaskWorkersPerSearchWorkerId, 0);
+  defaults->Set<int>(classic::SearchParams::kMiniBatchSizeId, 32);
+  defaults->Set<float>(classic::SearchParams::kCpuctId, 1.2f);
+  defaults->Set<float>(classic::SearchParams::kCpuctFactorId, 0.0f);
+  defaults->Set<float>(SharedBackendParams::kPolicySoftmaxTemp, 1.0f);
+  defaults->Set<int>(classic::SearchParams::kMaxCollisionVisitsId, 1);
+  defaults->Set<int>(classic::SearchParams::kMaxCollisionEventsId, 1);
+  defaults->Set<int>(classic::SearchParams::kCacheHistoryLengthId, 7);
+  defaults->Set<bool>(classic::SearchParams::kOutOfOrderEvalId, false);
+  defaults->Set<float>(classic::SearchParams::kTemperatureId, 1.0f);
+  defaults->Set<float>(classic::SearchParams::kNoiseEpsilonId, 0.25f);
+  defaults->Set<float>(classic::SearchParams::kFpuValueId, 0.0f);
+  defaults->Set<std::string>(SharedBackendParams::kHistoryFill, "no");
+  defaults->Set<std::string>(SharedBackendParams::kBackendId, "multiplexing");
+  defaults->Set<bool>(classic::SearchParams::kStickyEndgamesId, false);
+  defaults->Set<bool>(classic::SearchParams::kTwoFoldDrawsId, false);
+  defaults->Set<int>(classic::SearchParams::kTaskWorkersPerSearchWorkerId, 0);
 }
 
-SelfPlayTournament::SelfPlayTournament(
-    const OptionsDict& options,
-    CallbackUciResponder::BestMoveCallback best_move_info,
-    CallbackUciResponder::ThinkingCallback thinking_info,
-    GameInfo::Callback game_info, TournamentInfo::Callback tournament_info)
+SelfPlayTournament::SelfPlayTournament(const OptionsDict& options,
+                                       UciResponder* uci_responder,
+                                       GameInfo::Callback game_info,
+                                       TournamentInfo::Callback tournament_info)
     : player_options_{{options.GetSubdict("player1").GetSubdict("white"),
                        options.GetSubdict("player1").GetSubdict("black")},
                       {options.GetSubdict("player2").GetSubdict("white"),
                        options.GetSubdict("player2").GetSubdict("black")}},
-      best_move_callback_(best_move_info),
-      info_callback_(thinking_info),
+      uci_responder_(uci_responder),
       game_callback_(game_info),
       tournament_callback_(tournament_info),
       kTotalGames(options.Get<int>(kTotalGamesId)),
@@ -212,30 +210,32 @@ SelfPlayTournament::SelfPlayTournament(
     first_game_black_ = Random::Get().GetBool();
   }
 
+  static constexpr const char* kPlayerNames[2] = {"player1", "player2"};
+  static constexpr const char* kPlayerColors[2] = {"white", "black"};
+
   // Initializing networks.
-  for (const auto& name : {"player1", "player2"}) {
-    for (const auto& color : {"white", "black"}) {
+  std::vector<std::shared_ptr<Backend>> backend_list;
+  for (int name_idx : {0, 1}) {
+    for (int color_idx : {0, 1}) {
+      const auto& name = kPlayerNames[name_idx];
+      const auto& color = kPlayerColors[color_idx];
       const auto& opts = options.GetSubdict(name).GetSubdict(color);
-      const auto config = NetworkFactory::BackendConfiguration(opts);
-      if (networks_.find(config) == networks_.end()) {
-        networks_.emplace(config, NetworkFactory::LoadNetwork(opts));
+      for (const auto& backend : backend_list) {
+        if (backend->IsSameConfiguration(opts)) {
+          backends_[name_idx][color_idx] = backend;
+          break;
+        }
+      }
+      if (!backends_[name_idx][color_idx]) {
+        backends_[name_idx][color_idx] =
+            CreateMemCache(BackendManager::Get()->CreateFromParams(opts),
+                           options.GetSubdict(name));
+        backend_list.emplace_back(backends_[name_idx][color_idx]);
       }
     }
   }
 
-  // Initializing cache.
-  cache_[0] = std::make_shared<NNCache>(
-      options.GetSubdict("player1").Get<int>(kNNCacheSizeId));
-  if (kShareTree) {
-    cache_[1] = cache_[0];
-  } else {
-    cache_[1] = std::make_shared<NNCache>(
-        options.GetSubdict("player2").Get<int>(kNNCacheSizeId));
-  }
-
   // SearchLimits.
-  static constexpr const char* kPlayerNames[2] = {"player1", "player2"};
-  static constexpr const char* kPlayerColors[2] = {"white", "black"};
   for (int name_idx : {0, 1}) {
     for (int color_idx : {0, 1}) {
       auto& limits = search_limits_[name_idx][color_idx];
@@ -311,10 +311,7 @@ void SelfPlayTournament::PlayOneGame(int game_number) {
         player_options_[pl_idx][color].Get<bool>(kMoveThinkingId);
     // Populate per-player options.
     PlayerOptions& opt = options[color_idx[pl_idx]];
-    opt.network = networks_[NetworkFactory::BackendConfiguration(
-                                player_options_[pl_idx][color])]
-                      .get();
-    opt.cache = cache_[pl_idx].get();
+    opt.backend = backends_[pl_idx][color].get();
     opt.uci_options = &player_options_[pl_idx][color];
     opt.search_limits = search_limits_[pl_idx][color];
 
@@ -328,14 +325,14 @@ void SelfPlayTournament::PlayOneGame(int game_number) {
       }
       // In non-verbose mode, output the last "info" message.
       if (!verbose_thinking && !last_thinking_info.empty()) {
-        info_callback_(last_thinking_info);
+        uci_responder_->OutputThinkingInfo(&last_thinking_info);
         last_thinking_info.clear();
       }
       BestMoveInfo rich_info = info;
       rich_info.player = pl_idx + 1;
       rich_info.is_black = player1_black ? pl_idx == 0 : pl_idx != 0;
       rich_info.game_id = game_number;
-      best_move_callback_(rich_info);
+      uci_responder_->OutputBestMove(&rich_info);
     };
 
     opt.info_callback =
@@ -348,7 +345,7 @@ void SelfPlayTournament::PlayOneGame(int game_number) {
             info.game_id = game_number;
           }
           if (verbose_thinking) {
-            info_callback_(rich_info);
+            uci_responder_->OutputThinkingInfo(&rich_info);
           } else {
             // In non-verbose mode, remember the last "info" messages.
             last_thinking_info = std::move(rich_info);
@@ -452,12 +449,8 @@ void SelfPlayTournament::PlayMultiGames(int game_id, size_t game_count) {
   }
 
   PlayerOptions options[2];
-  options[0].network =
-      networks_[NetworkFactory::BackendConfiguration(player_options_[0][0])]
-          .get();
-  options[1].network =
-      networks_[NetworkFactory::BackendConfiguration(player_options_[1][1])]
-          .get();
+  options[0].backend = backends_[0][0].get();
+  options[1].backend = backends_[1][1].get();
 
   std::list<std::unique_ptr<MultiSelfPlayGames>>::iterator game1_iter;
   auto aborted = false;
@@ -473,12 +466,8 @@ void SelfPlayTournament::PlayMultiGames(int game_id, size_t game_count) {
   // PLAY GAMEs!
   if (!aborted) game1.Play();
 
-  options[0].network =
-      networks_[NetworkFactory::BackendConfiguration(player_options_[0][1])]
-          .get();
-  options[1].network =
-      networks_[NetworkFactory::BackendConfiguration(player_options_[1][0])]
-          .get();
+  options[0].backend = backends_[0][1].get();
+  options[1].backend = backends_[1][0].get();
 
   std::list<std::unique_ptr<MultiSelfPlayGames>>::iterator game2_iter;
   {
@@ -658,9 +647,9 @@ void SelfPlayTournament::SaveResults() {
   if (kTournamentResultsFile.empty()) return;
   std::ofstream output(kTournamentResultsFile, std::ios_base::app);
   auto p1name =
-      player_options_[0][0].Get<std::string>(NetworkFactory::kWeightsId);
+      player_options_[0][0].Get<std::string>(SharedBackendParams::kWeightsId);
   auto p2name =
-      player_options_[1][0].Get<std::string>(NetworkFactory::kWeightsId);
+      player_options_[1][0].Get<std::string>(SharedBackendParams::kWeightsId);
 
   output << std::endl;
   output << "[White \"" << p1name << "\"]" << std::endl;
